"""nodoctest
all.py -- much of sage is imported into this module, so you don't
          have to import everything individually.  This is what
          is imported by IPython when sage starts.
"""

#*****************************************************************************
#
#   SAGE:  System for Arithmetic Geometry Experimentation    
#
#       Copyright (C) 2005 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************



#print """
#**********************************************************
#*  SAGE Version 0.3                                      *
#*  System for Arithmetic Geometry Experimentation        *
#*  (c) William Stein, 2005                               *
#*  http://modular.fas.harvard.edu/SAGE                   *
#*  Distributed under the terms of the GPL                *
#**********************************************************
#Help: object? -> Documentation about 'object'.
#"""

import os
from math import sin, log, cos, tan


from sage.rings.rings import *

#from sage.linalg.linalg import *
from sage.linalg.matrix import *
from sage.linalg.matrix_space import *
from sage.linalg.vector import *
from sage.linalg.vector_space import *
from sage.misc.misc import *
from sage.modular.modsym.modsym import *
from sage.modular.modsym.heilbronn import *
from sage.modular.modform.modform import *
from sage.modular.abvar.modabvar import *
from sage.modular.cusps import *
from sage.modular.modsym.p1list import *
from sage.rings.rings import *
from sage.algebras.algebras import *
import sage.rings.integer_ring
from sage.ellcurve.ellcurve import *
from sage.modular.dirichlet import *
from sage.modular.congroup import *
from sage.functions.transcendental import *
from sage.groups.abelian import *
import sage.pari.py_pari as py_pari
pari = py_pari.pari
import sage.tables.all as tables

from sage.rings.arith import *

from os import system

# maximize memory resources
try:
    import resource   # unix only...
    resource.setrlimit(resource.RLIMIT_AS, (-1,-1))
except:
    pass

# abreviations
EC = EllipticCurve
IM = IntegerModRing
Integers = IM
MF = ModularForms
MS = ModularSymbols
NF = NumberField
PR = PolynomialRing
VS = VectorSpace



def CS(x):
    return x.cuspidal_subspace()

def NS(x):
    return x.new_subspace()

def OS(x):
    return x.old_subspace()

def ES(x):
    return x.eisenstein_subspace()

def NCMS(n,k=2,sign=1):
    return ms(n,weight=k,sign=sign).cuspidal_subspace().new_subspace()

quo = rings.Rational
tm = misc.cputime

Q = rings.RationalField()
Z = rings.IntegerRing()
R = rings.RealField()
C = rings.ComplexField()
I = C.gen(0); i = C.gen(0)

# Macaulay2 notation:
QQ = Q; ZZ = Z; RR = R; CC = C


def rprint(x):
    for z in x:
        print z

def pr(x):
    for z in x:
        print z

##############################################################################
# There are many functions on elements of a ring, which mathematicians
# usually write f(x), e.g., it is weird to write x.log() and natural
# to write log(x).  The functions below allow for the more familiar syntax.
##############################################################################
def arg(x):
    try: return x.arg()
    except AttributeError: return C(x).arg()

def base_ring(x): return x.base_ring()
def base_field(x): return x.base_field()

def basis(x): return x.basis()

def charpoly(x): return x.charpoly()

def decomposition(x): return x.decomposition()

def det(x): return x.det()

def exp(x):
    try: return x.exp()
    except AttributeError: return math.exp(x)

def factor(x):
    try: return x.factor()
    except AttributeError: return sage.rings.integer_ring.factor(x)

def gens(x): return x.gens()

def hecke_operator(x,n): return x.hecke_operator(n)

def image(x): return x.image()

def j_invariant(x): return x.j_invariant()

def kernel(x): return x.kernel()

def log(x,b=None):
    """
    The log of x to the base b.  The default base is e.

    INPUT:
        x -- number
        b -- base (default: None, which means natural log)
    OUTPUT:
        number

    WARNING: In MAGMA, the order of arguments is reversed, so the
    base is given first.   We use the opposite ordering, so the
    base can be viewed as an optional second argument.
    """
    if b is None:
        try: return x.log()
        except AttributeError:
            return math.log(x)
    else:
        try: return x.log(b)
        except AttributeError:
            return math.log(x)/math.log(b)
    

def ngens(x): return x.ngens()

def norm(x): return x.norm()

def order(x): return x.order()

def parent(x): return x.parent()

def rank(x): return x.rank()

def real(x):
    try: return x.real()
    except AttributeError: return C(x).real()

def imag(x): 
    try: return x.imag()
    except AttributeError: return C(x).imag()

def sqrt(x):
    try: return x.sqrt()
    except AttributeError: return math.sqrt(x)



##########################################################################
# I tried this save/restore session, but it seems hopeless.
# For example, how to restore the local caches of modular symbols
# spaces and vector spaces to the modules?  It's probably better
# to have an object-by-object database anyways.

# def save_session(dirname):
#     """
#     Save the current session to the named directory.  A file is
#     created for each object currently defined.
#     """
#     if os.path.exists(dirname):
#         os.rename(dirname,dirname+".old")
#     os.mkdir(dirname)
#     for x in show_identifiers():
#         db.save(globals()[x], "%s/%s"%(dirname,x))

# import posix
# def load_session(dirname):
#     """
#     Load all variables defined in a session from the named directory.
#     """
#     assert os.path.exists(dirname), "Directory %s does not exist."%dirname
#     for x in posix.listdir(dirname):
#         globals()[x] = db.load("%s/%s"%(dirname,x))

        
iglob = globals().keys()[:] + \
        ['os', '__', '_i1', '_i', '_ii', '___', 'show_identifiers', '_iii', '_', 'iglob']
def show_identifiers():
    """
    Returns a list of the names of non-hidden (i.e., not starting with
    an underscore) identifiers defined by the user in this session.
    These are the identifiers saved by the save command.
    """
    return [k for k in globals().keys() if not k in iglob and k[0] != "_"]


