"""
Abelian groups
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.misc.misc
import sage.rings.arith as arith

class AbelianGroup:
    def __init__(self, invariants):
        invariants.sort()
        invariants = [x for x in invariants if x != 1]
        self.__invariants = invariants

    def __repr__(self):
        return "Abelian Group with invariants %s"%self.__invariants

    def invariants(self):
        return self.__invariants

    def order(self):
        try:
            return self.__len
        except AttributeError:
            self.__len = sage.misc.misc.mul(self.__invariants)
        return self.__len

    def exponent(self):
        return arith.LCM(self.__invariants)

               


    
