"""
A pexpect interface to mathematica.

This interface is extremely flexible, since it's exactly like typing
into the mathematica interpreter, and anything that works there should
work here.  Unfortunately, it's sometimes much slower than using
C-library calls, and it's very unlikely to work on, e.g., windows.
Also, the pexpect module must be installed, which is not standard with
Python, and "math" must be in your path.

"""



#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import pexpect
_MATH = "math"
_PROMPT = ":="

_objsMath = {}
class _uniqMath(object):
    def __new__(cls):
        if _objsMath.has_key(0):
            return _objsMath[0]
        R = object.__new__(cls)
        R.restart()
        _objsMath[0] = R
        return R

class Math(_uniqMath):
    def __init__(self):
        pass
    
    def restart(self):
        if hasattr(self, "__expect"):
            self.__expect.close()
        self.__expect = pexpect.spawn(_MATH)
        self.__expect.timeout = 999999999
        self.__expect.expect(_PROMPT)
        #self.__expect.expect(_PROMPT)
        #self.__expect.expect(_PROMPT)
        
    def cmd(self, line, raw=True):
        line = line.replace("\n",";")
        #print "line = ", line
        self.__expect.sendline(line)
        self.__expect.expect(_PROMPT)
        out = self.__expect.before
        #print "out = ", out
        if out.find("  ***")!=-1 and out.find("^")!=-1:
            raise RuntimeError, "\n"+out[out.find("  ***"):]
        i = out.find("\n"); 
        out = out[i+1:].replace("\r\n","\n")
        if raw: return out[:-1]
        j = out.find("=")
        return out[j+2:-2].replace("\r\n","\n")
    
    def __call__(self, line, raw=True):
        return self.cmd(line, raw=raw)

import sage.rings.rings as rings
I = rings.ComplexField().gen()
def eval_num(s):
    if s.find(" 10   ") == -1:
        i = s.find("=")
        s = s[i+1:]
        i = s.find("\n")
        s = s[:i]
        s = s.replace(" I", "*I")
        return eval(s)
    else:  # has exponents
        s = s[1:]  # get rid of first carriage return
        i = s.find('\n')
        # exps contains the one or two exponents
        exps = [eval(x) for x in s[:i].split()]
        i = s.find("=")
        s = s[i+1:]
        i = s.find("\n")
        s = s[:i]
        s = s.replace(" I", "*I")
        i = s.find(" 10")
        s = s[:i] + "e%s"%exps[0] + s[i+4:]
        i = s.find(" 10")
        if i != -1:
            s = s[:i] + "e%s"%exps[1] + s[i+4:]
        return eval(s)
