"""
Script to create shared library associated to pyrex file.
"""

from distutils.core import setup
from distutils.extension import Extension
import os
import sys

DEBUG=True
HOME=os.environ["HOME"]

# Pyrex makes some messy C code so limit some warnings when we know how.
import distutils.sysconfig
if distutils.sysconfig.get_config_var('CC').startswith("gcc"):
    pyrex_compile_options = [
        '-Wno-unused-label', '-Wno-unused-function', 
        '-Wno-parentheses', '-Wno-unused-variable']
else:
    pyrex_compile_options = []

def build(modules, libs=[], nowarn=False, opts=[], link_opts=[], run_pyrex=True):
    if DEBUG:
        opts.append("-g")
    if isinstance(modules, str): modules = [modules]
    if nowarn: pyrex_compile_options.append("-w")
    for x in opts:
        pyrex_compile_options.append(x)
    module = modules[0]
    
    sources = ['%s.c'%mod for mod in modules ]
    
    cmd = "%s/local/bin/pyrexc %s.pyx"%(HOME,module)
    if run_pyrex:
        i = os.system(cmd)
    else:
        i = 0
    if i == 0:
        setup(name=module, ext_modules=[
            Extension(module,
                      sources=sources,
                      extra_compile_args = pyrex_compile_options, \
                      extra_link_args = ["-l"+lib for lib in libs] + link_opts) ])
        os.system("mv %s/local/lib/python2.4/site-packages/%s.so ."%(HOME,module))
