"""
Congruence subgroups of SL_2(Z)
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.misc.errors as errors

class CongruenceSubgroup:
    def __init__(self, level):
        level=int(level)
        if level <= 0:
            raise errors.UsageError("Congruence groups only defined for positive levels.")
        self.__level = level

    def __hash__(self):
        return hash(self.name())

    def are_equivalent(self, x, y):
        raise NotImplementedError

    def coset_reps(self):
        raise NotImplementedError

    def generators(self):
        raise NotImplementedError

    def level(self):
        return self.__level

    def __cmp__(self, right):
        raise NotImplementedError

    def name(self):
        raise NotImplementedError


class Gamma0(CongruenceSubgroup):
    def __init__(self, level):
        CongruenceSubgroup.__init__(self, level)

    def __repr__(self):
        return "Gamma0(%s)"%self.level()

    def name(self):
        return "g0%s"%self.level()

    def __cmp__(self, right):
        if not isinstance(right, Gamma0):
            return -1
        if self.level() < right.level():
            return -1
        elif self.level() > right.level():
            return 1
        return 0

class Gamma1(CongruenceSubgroup):
    def __init__(self, level):
        CongruenceSubgroup.__init__(self, level)

    def __repr__(self):
        return "Gamma1(%s)"%self.level()

    def name(self):
        return "g1%s"%self.level()

    def __cmp__(self, right):
        if not isinstance(right, Gamma1):
            return -1
        if self.level() < right.level():
            return -1
        elif self.level() > right.level():
            return 1
        return 0

class GammaH(CongruenceSubgroup):
    def __init__(self, level, H):
        CongruenceSubgroup.__init__(self, level)
        self.__H = H

    def __repr__(self):
        return "Gamma_H(%s,H=%s)"%(self.level(), self.__H)

    def name(self):
        return "gh%s_%s"%(self.level(), self.__H)
