"""
Ambient spaces of modular symbols with a nontrivial Dirichlet character.
"""

#*****************************************************************************
#       MANIN: Modular Forms Software, (c) William Stein, 2004
#
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import weakref

import ambient
import space
import sage.modular.congroup as congroup

class ModularSymbolsSubspace(space.ModularSymbolsSpace):
    
    def __init__(self, ambient_space, vector_space):
        self.__ambient_space = ambient_space
        self.__vector_space = vector_space
        A = ambient_space
        space.ModularSymbolsSpace.__init__(self, A.group(), A.weight(), \
                                     A.character(), A.sign(), A.base_field())

    def __repr__(self):
        return "Subspace of Modular Symbols of dimension %s and level %s"%(
                    self.dimension(), self.level())

    def _compute_hecke_matrix(self, n):
        A = self.ambient_space().hecke_matrix(n)
        T = A.restrict(self.embedded_vector_space())
        self._hecke_matrices[n] = T
        return T

    def ambient_space(self):
        return self.__ambient_space

    def ambient_vector(self, v):
        """
        Given a vector v in self.vector_space(), return the
        corresponding vector in the ambient modular symbols space.
        """
        B = self.__vector_space.basis()
        zero = self.__vector_space.ambient()(0)
        return sum([v[i]*B[i] for i in range(len(B))], zero)

    def dimension(self):
        return self.__vector_space.dimension()

    def embedded_vector_space(self):
        return self.__vector_space
        
    def is_ambient(self):
        return False



