# Buhler-Gross algorithm for computing L-series derivatives

# Based on an implementation in GP by tom@womack.net and
# jec@maths.nottingham.ac.uk

# This is practical for any rank and conductors <= 10^10.

# Section 0: prime-number utility which really should be
# built-in. This version due to Karim (reference for upper and lower
# bounds is Rosser & Schoenfeld, Approximate formulas for some functions
# of prime numbers, Illinois J Math 6 (1962) pp 64-94)
from sage.pari.func import *
ONE = pari(1)
TWO = pari(2)
THREE = pari(3)

def pith(x):
    x = pari(x)  # make type correct

    if x <= 2: return x == 2
    lx = log(x)
    l = floor (x / lx)
    r = floor(l * (ONE + THREE/(TWO*lx)))
    while r - l > 1:
        m = (l+r) >> 1
        if prime(m) <= x:
            l = m
        else:
            r = m
    return l

# Section 1: G_r

# We need a load of Magic Constants, which are the coefficients
# of the Taylor expansion of GAMMA(1+s). Now, [BGZ] we have
# log(Gamma(1+s)) = -gamma s + sum(n=2,infinity,(-1)^n*zeta(n)/n)
# and Pari can do polynomial exponential

s = pari("s")
ellld_ll = -euler()*s + sum([(-1)**n * zeta(n) / n * s**n for n in xrange(2,31)])
ellld_mm = exp(ellld_ll)
ellld_pn_coeffs = vector([polcoeff(ellld_mm,i) for i in xrange(0,25)])

def ellld_P(r,t):
    global ellld_pn_coeffs
    print ellld_pn_coeffs
    print [ellld_pn_coeffs[r-m] for m in xrange(0,r)]
    return sum([ellld_pn_coeffs[r-m+1] * t**m / factorial(m) for m in xrange(0,r+1)])
