"""
A pexpect interface to gp interpreter.

This interface is extremely flexible, since it's exactly like typing
into the PARI interpreter, and anything that works there should work
here.  Unfortunately, it's sometimes much slower than using C-library
calls, and it's very unlikely to work on, e.g., windows.  Also, the
pexpect module must be installed, which is not standard with Python,
and gp must be in your path.

"""



#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import pexpect
_PARI = "gp -q --emacs"
_PROMPT = "\?"

_objsPari = {}
class _uniqPari(object):
    def __new__(cls):
        if _objsPari.has_key(0):
            return _objsPari[0]
        R = object.__new__(cls)
        R.restart()
        _objsPari[0] = R
        return R

class Pari(_uniqPari):
    def __init__(self):
        pass
    
    def restart(self):
        if hasattr(self, "__expect"):
            self.__expect.close()
        self.__expect = pexpect.spawn(_PARI)
        self.__expect.timeout = 999999999
        self.__expect.expect(_PROMPT)
        #self.__expect.expect(_PROMPT)
        #self.__expect.expect(_PROMPT)
        
    def cmd(self, line, raw=True):
        line = line.replace("\n",";")
        self.__expect.sendline(line)
        self.__expect.expect(_PROMPT)
        out = self.__expect.before
        if out.find("  ***")!=-1 and out.find("^")!=-1:
            raise RuntimeError, "\n"+out[out.find("  ***"):]
        i = out.find("\n"); 
        out = out[i+1:].replace("\r\n","\n")
        if raw: return out[:-1]
        j = out.find("=")
        return out[j+2:-2].replace("\r\n","\n")
    
    def __call__(self, line, raw=True):
        return self.cmd(line, raw=raw)
