/*-*- compile-command: "/usr/bin/gcc -c -o rank.gp.o -O3 -DGCC_INLINE -Wall -fno-strict-aliasing -fomit-frame-pointer -I/usr/local/include rank.gp.c && /usr/bin/gcc -o rank.gp.so -shared -O3 -DGCC_INLINE -Wall -fno-strict-aliasing -fomit-frame-pointer -Wl,-shared rank.gp.o -lc -ldl -lm -L/usr/local/lib -lpari"; -*-*/
#include <pari/pari.h>
/*
GP;install("pith","D0,G,p","pith","./rank.gp.so");
GP;install("ellld_P","D0,G,D0,G,p","ellld_P","./rank.gp.so");
GP;install("ellld_G","D0,G,D0,G,p","ellld_G","./rank.gp.so");
GP;install("ellld_getap","D0,G,","ellld_getap","./rank.gp.so");
GP;install("ellld_getp","D0,G,","ellld_getp","./rank.gp.so");
GP;install("BGadd","vD0,G,D0,G,D0,G,D0,G,p","BGadd","./rank.gp.so");
GP;install("elllderiv","D0,G,D0,G,D0,G,p","elllderiv","./rank.gp.so");
GP;install("ellanalyticrank","D0,G,p","ellanalyticrank","./rank.gp.so");
GP;install("ellRegSha","D0,G,D0,G,p","ellRegSha","./rank.gp.so");
GP;install("ellSha","D0,G,D0,G,p","ellSha","./rank.gp.so");
GP;install("init_rank","vp","init_rank","./rank.gp.so");
*/
GEN pith(GEN x, long prec);
GEN ellld_P(GEN r, GEN t, long prec);
GEN ellld_G(GEN r, GEN x, long prec);
GEN ellld_getap(GEN r);
GEN ellld_getp(GEN r);
void BGadd(GEN n, GEN i, GEN a, GEN last_a, long prec);
GEN elllderiv(GEN e, GEN r, GEN bnd, long prec);
GEN ellanalyticrank(GEN e, long prec);
GEN ellRegSha(GEN e, GEN r, long prec);
GEN ellSha(GEN e, GEN pts, long prec);
void init_rank(long prec);
/*End of prototype*/

static GEN primelimit;
static GEN seriesprecision;
static GEN ellld_ll;
static GEN s;
static GEN ellld_mm;
static GEN ellld_pn_coeffs;
/*End of global vars*/

void 
init_rank(long prec)
{
  GEN p1;
  GEN p2;	  /* vec */
  primelimit = polx[fetch_user_var("primelimit")];
  seriesprecision = polx[fetch_user_var("seriesprecision")];
  ellld_ll = polx[fetch_user_var("ellld_ll")];
  s = polx[fetch_user_var("s")];
  ellld_mm = polx[fetch_user_var("ellld_mm")];
  ellld_pn_coeffs = polx[fetch_user_var("ellld_pn_coeffs")];
  /* Buhler-Gross algorithm for computing L-series derivatives */
  
  /* Implementation by tom@womack.net */
  /* Repaired and improved by jec@maths.nottingham.ac.uk */
  
  /* This is practical for any rank and conductors <= 10^10 in 10 */
  /* minutes per L' calculation (on P3/500E). */
  
  default0(GSTR(GENtoGENstr(primelimit)), GSTR(GENtoGENstr(stoi(2000000))), 0);
  /* Section 1: G_r */
  
  /* We need a load of Magic Constants, which are the coefficients */
  /* of the Taylor expansion of GAMMA(1+s). Now, [BGZ] we have */
  /* log(Gamma(1+s)) = -gamma s + sum(n=2,infinity,(-1)^n*zeta(n)/n) */
  /* and Pari can do polynomial exponential */
  
  default0(GSTR(GENtoGENstr(seriesprecision)), GSTR(GENtoGENstr(stoi(30))), 0);
  p1 = gzero;
  {
    long n;
    for (n = 2; n <= 30; ++n)
      p1 = gadd(p1, gmul(gdivgs(gmul(gpowgs(stoi(-1), n), gzeta(stoi(n), prec)), n), gpowgs(s, n)));
  }
  ellld_ll = gadd(gmul(negr(mpeuler(prec)), s), p1);
  ellld_mm = gexp(ellld_ll, prec);
  {
    long i;
    p2 = cgetg(26, t_VEC);
    for (i = 1; i <= 25; ++i)
      p2[i] = (long) polcoeff0(ellld_mm, i - 1, -1);
  }
  ellld_pn_coeffs = p2;
  return;
}

/* Section 0: prime-number utility which really should be */
/* built-in. This version due to Karim (reference for upper and lower */
/* bounds is Rosser & Schoenfeld, Approximate formulas for some functions */
/* of prime numbers, Illinois J Math 6 (1962) pp 64-94) */

GEN 
pith(GEN x, long prec)
{
  GEN p1;
  GEN l;
  GEN r;
  GEN lx;
  GEN m = polx[fetch_user_var("m")];
  if (gcmpgs(x, 2) <= 0)
  {
    long l2;	  /* bool */
    l2 = gegalgs(x, 2);
    return stoi(l2);
  }
  lx = glog(x, prec);
  l = gfloor(gdiv(x, lx));
  r = gfloor(gmul(l, gaddsg(1, gdivsg(3, gmulsg(2, lx)))));
  while (gcmpgs(gsub(r, l), 1) > 0)
  {
    m = gshift(gadd(l, r), -1);
    if (gcmp(prime(itos(m)), x) <= 0)
      l = m;
    else
      r = m;
  }
  p1 = l;
  return p1;
}

GEN 
ellld_P(GEN r, GEN t, long prec)
{
  GEN p1;
  GEN m;
  GEN p2;
  p2 = gzero;
  {
    GEN m;
    long l3;
    for (m = gzero; gcmp(m, r) <= 0; m = gaddgs(m, 1))
    {
      l3 = itos(gaddgs(gsub(r, m), 1));
      p2 = gadd(p2, gdiv(gmul((GEN) ellld_pn_coeffs[l3], gpow(t, m, prec)), mpfactr(itos(m), prec)));
    }
  }
  p1 = p2;
  return p1;
}

GEN 
ellld_G(GEN r, GEN x, long prec)
{
  GEN p1;
  GEN ss;
  GEN oldss;
  GEN n;
  GEN sn;
  if (gcmpgs(x, 30) > 0)
    return gzero;
  ss = ellld_P(r, glog(ginv(x), prec), prec);
  oldss = gsubgs(ss, 1);
  n = gun;
  sn = gneg(gpow(stoi(-1), r, prec));
  while (gcmp(gabs(gsub(ss, oldss), prec), flisexpr("1e-15")) > 0)
  {
    oldss = ss;
    ss = gadd(ss, gdiv(gmul(sn, gpow(x, n, prec)), gmul(gpow(n, r, prec), mpfactr(itos(n), prec))));
    n = gaddgs(n, 1);
    sn = gneg(sn);
  }
  p1 = gmulsg(2, ss);
  return p1;
}

/* Section 2: the BG recursion */
/* Some global variables are necessary, but I've named them */
/* to try to avoid contaminating the namespace *too* much. */

GEN 
ellld_getap(GEN r)
{
  GEN p1;
  GEN p2;
  GEN ellld_pnum = polx[fetch_user_var("ellld_pnum")];
  GEN ellld_ap = polx[fetch_user_var("ellld_ap")];
  GEN est = polx[fetch_user_var("est")];
  if (gcmp(r, ellld_pnum) <= 0)
    p2 = (GEN) ellld_ap[itos(r)];
  else
    p2 = ellap0(est, prime(itos(r)), 0);
  p1 = p2;
  return p1;
}

GEN 
ellld_getp(GEN r)
{
  GEN p1;
  GEN p2;
  GEN ellld_pnum = polx[fetch_user_var("ellld_pnum")];
  GEN ellld_p = polx[fetch_user_var("ellld_p")];
  if (gcmp(r, ellld_pnum) <= 0)
    p2 = (GEN) ellld_p[itos(r)];
  else
    p2 = prime(itos(r));
  p1 = p2;
  return p1;
}

void 
BGadd(GEN n, GEN i, GEN a, GEN last_a, long prec)
{
  GEN j;
  GEN j0;
  GEN next_a;
  GEN ellld_sum = polx[fetch_user_var("ellld_sum")];
  GEN ellld_r = polx[fetch_user_var("ellld_r")];
  GEN ellld_X = polx[fetch_user_var("ellld_X")];
  GEN ellld_bnd = polx[fetch_user_var("ellld_bnd")];
  GEN ellld_N = polx[fetch_user_var("ellld_N")];
  if (gcmp0(a))
    j0 = i;
  else
  {
    ellld_sum = gadd(ellld_sum, gdiv(gmul(a, ellld_G(ellld_r, gmul(ellld_X, n), prec)), n));
    j0 = gun;
  }
  j = j0;
  while ((gcmp(j, i) <= 0) && (gcmp(gmul(ellld_getp(j), n), ellld_bnd) <= 0))
  {
    next_a = gmul(a, ellld_getap(j));
    if (gegal(j, i) && !gegalgs(gmod(ellld_N, ellld_getp(j)), 0))
      next_a = gsub(next_a, gmul(ellld_getp(j), last_a));
    BGadd(gmul(ellld_getp(j), n), j, next_a, a, prec);
    j = gaddgs(j, 1);
  }
  return;
}

GEN 
elllderiv(GEN e, GEN r, GEN bnd, long prec)
{
  GEN p1;
  GEN cached_pmax;
  GEN loop_p_num;
  GEN ithresh;
  GEN N;
  GEN est;
  GEN ellld_pnum = polx[fetch_user_var("ellld_pnum")];
  GEN ellld_p = polx[fetch_user_var("ellld_p")];
  GEN ellld_ap = polx[fetch_user_var("ellld_ap")];
  GEN p2;	  /* vec */
  GEN ellld_N = polx[fetch_user_var("ellld_N")];
  GEN ellld_X = polx[fetch_user_var("ellld_X")];
  GEN ellld_r = polx[fetch_user_var("ellld_r")];
  GEN ellld_bnd = polx[fetch_user_var("ellld_bnd")];
  GEN ellld_sum = polx[fetch_user_var("ellld_sum")];
  est = initell(e, prec);
  cached_pmax = gsqrt(bnd, prec);
  ellld_pnum = pith(cached_pmax, prec);
  loop_p_num = pith(bnd, prec);
  ellld_p = primes(itos(ellld_pnum));
  {
    long i;
    p2 = cgetg(itos(ellld_pnum)+1, t_VEC);
    for (i = 1; gcmpsg(i, ellld_pnum) <= 0; ++i)
      p2[i] = (long) ellap0(est, (GEN) ellld_p[i], 0);
  }
  ellld_ap = p2;
  /* Version 2.0.19 has acknowledged bug in ellap for p=2 */
  ellld_ap[1] = (long) akell(est, gdeux);
  N = (GEN) globalreduction(e)[1];
  ellld_N = N;
  ellld_X = gdiv(mulsr(2, mppi(prec)), gsqrt(N, prec));
  ellld_r = r;
  ellld_bnd = bnd;
  ellld_sum = ellld_G(r, ellld_X, prec);
  ithresh = gdeux;
  {
    GEN i;
    for (i = gun; gcmp(i, loop_p_num) <= 0; i = gaddgs(i, 1))
      /*    if(i==ithresh, */
      /*      print([i,ellld_sum]); */
      /*      ithresh=ithresh*2; */
      /*    ); */
      BGadd(ellld_getp(i), i, ellld_getap(i), gun, prec);
  }
  p1 = ellld_sum;
  return p1;
}

GEN 
ellanalyticrank(GEN e, long prec)	  /* vec */
{
  GEN p1;	  /* vec */
  GEN rksgn;
  GEN ZB;
  GEN egrd;
  GEN cond;
  GEN localbnd;
  GEN Lr1;
  GEN RS;
  GEN om;
  GEN p2;	  /* vec */
  GEN p3;	  /* vec */
  GEN p4;	  /* vec */
  egrd = globalreduction(e);
  /* We had to compute this anyway to get the conductor, and */
  /* ellap gives wrong answers on non-minimal curves ... */
  p2 = cgetg(5, t_VEC);
  p2[1] = (long) gun;
  p2[2] = (long) gzero;
  p2[3] = (long) gzero;
  p2[4] = (long) gzero;
  if (!gegal((GEN) egrd[2], p2))
  {
    pariputsf("INPUT CURVE NOT MINIMAL\n");
    e = coordch(e, (GEN) egrd[2]);
    p3 = cgetg(6, t_VEC);
    p3[1] = e[1];
    p3[2] = e[2];
    p3[3] = e[3];
    p3[4] = e[4];
    p3[5] = e[5];
    pariputsf("MINIMISED TO %Z\n",p3);
  }
  cond = (GEN) egrd[1];
  /* No, I can't justify this, but it should be big enough ... */
  localbnd = ground(gmulsg(2, gsqrt(cond, prec)));
  pariputsf("Summing %Z a_n terms\n",localbnd);
  rksgn = stoi(ellrootno(e, NULL));
  if (gcmp_1(rksgn))
    pariputsf("Rank is odd\n");
  else
    pariputsf("Rank is even\n");
  ZB = gdivgs(gsubsg(1, rksgn), 2);
  Lr1 = gzero;
  /* And I'm sure it's conjectural that this is close-enough to 0 */
  while (gcmp(gabs(Lr1, prec), flisexpr("0.001")) < 0)
  {
    Lr1 = elllderiv(e, ZB, localbnd, prec);
    pariputsf("L^(%Z)=%Z\n",ZB,Lr1);
    ZB = gaddgs(ZB, 2);
  }
  /* Also compute the quantity reg*sha from the BSD conjecture */
  om = (GEN) e[15];
  if (gcmpgs(member_disc(e), 0) > 0)
    om = gmulsg(2, om);
  p4 = cgetg(4, t_VEC);
  p4[1] = (long) gsubgs(ZB, 2);
  p4[2] = lcopy(Lr1);
  p4[3] = (long) gdiv(gmul(Lr1, gsqr((GEN) elltors0(e, 0)[1])), gmul(om, (GEN) egrd[3]));
  p1 = p4;
  return p1;
}

GEN 
ellRegSha(GEN e, GEN r, long prec)
{
  GEN p1;
  GEN egrd;
  GEN om;
  GEN Lfn;
  om = (GEN) e[15];
  if (gcmpgs(member_disc(e), 0) > 0)
    om = gmulsg(2, om);
  egrd = globalreduction(e);
  Lfn = elllderiv(e, r, gmulsg(2, gsqrt((GEN) egrd[1], prec)), prec);
  p1 = gdiv(gmul(Lfn, gsqr((GEN) elltors0(e, 0)[1])), gmul(om, (GEN) egrd[3]));
  return p1;
}

GEN 
ellSha(GEN e, GEN pts, long prec)
{
  GEN p1;
  p1 = gdiv(ellRegSha(e, stoi(glength(pts)), prec), det(mathell(e, pts, prec)));
  return p1;
}

