"""
Interface to gp interpreter
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import os

import sage.pari.py_pari as py_pari

_pari_id = 0

class PariObject(object):
    def __init__(self, x):
        global _pari_id
        _pari_id += 1
        self.id = _pari_id
        #try:
        if isinstance(x, PariObject):
            py_pari.execute("v%s=v%s;"%(self.id, x.id))
        else:
            py_pari.execute("v%s=%s;"%(self.id, x))
        #except RuntimeError:
        #    raise ValueError, "unable to create PARI object from %s"%x
            
    def __del__(self):
        # todo -- figure out how to free variables...
        pass
        #py_pari.execute("kill(v%s);"%self.id)
        
    def __repr__(self):
        return py_pari.eval("v%s;"%self.id)

    def __str__(self):
        return py_pari.eval("v%s;"%self.id)

    def __int__(self):
        return int(str(self))

    def __long__(self):
        return long(str(self))
    
    def __float__(self):
        return float(str(self))

    def list(self):
        """
        If possible, convert PARI object to a Python list of
        Python objects.
        """
        s = str(self)
        if s[:4] == "Mat(":
            s = "[" + s[4:-1] + "]"
        s = s.replace("~","")
        if s.find(";") != -1:
            s = s.replace(";","], [")
            s = "[" + s + "]"
            return eval(s)
        else:
            return eval(s)

    def __getitem__(self, t):
        try:
            if isinstance(t, tuple):
                x = ",".join([str(n+1) for n in t])
            else:
                t = int(t)
                x = t+1
            s = "v%s[%s]"%(self.id, x)
            return PariObject(s)
        except ValueError:
            raise IndexError, "invalid index "+n
        
    def __setitem__(self, t, val):
        try:
            if isinstance(t, int):
                x = t+1
            else:
                x = ",".join([str(n+1) for n in t])
            s = "v%s[%s] = v%s;"%(self.id, x, PariObject(val).id)
            py_pari.execute(s)
        except ValueError:
            raise IndexError, "invalid index "+n
            
    def __getattr__(self, cmdname):
        def f(*args):
            cmd = "%s(v%s,"%(cmdname,self.id)
            for v in args:
                if not isinstance(v, PariObject):
                    x = PariObject(v)
                else:
                    x = v
                cmd += "v%s"%x.id + ","
            cmd = cmd[:-1] + ")"
            return PariObject(cmd)
        self.__dict__[cmdname] = f
        return f  
        
    def __add__(self, right):
        if not isinstance(right, PariObject):
            right = PariObject(right)
        return PariObject("v%s + v%s"%(self.id, right.id))
        
    def __radd__(self, left):
        if not isinstance(left, PariObject):
            left = PariObject(left)
        return PariObject("v%s + v%s"%(left.id, self.id))
        
    def __mul__(self, right):
        if not isinstance(right, PariObject):
            right = PariObject(right)
        return PariObject("v%s*v%s"%(self.id, right.id))

    def __rmul__(self, left):
        if not isinstance(left, PariObject):
            left = PariObject(left)
        return PariObject("v%s * v%s"%(left.id, self.id))
        
    def __sub__(self, right):
        if not isinstance(right, PariObject):
            right = PariObject(right)
        return PariObject("v%s - v%s"%(self.id, right.id))
        
    def __rsub__(self, left):
        if not isinstance(left, PariObject):
            left = PariObject(left)
        return PariObject("v%s - v%s"%(left.id, self.id))

    def __div__(self, right):
        if not isinstance(right, PariObject):
            right = PariObject(right)
        return PariObject("v%s / v%s"%(self.id, right.id))

    def __rdiv__(self, left):
        if not isinstance(left, PariObject):
            left = PariObject(left)
        return PariObject("v%s / v%s"%(left.id, self.id))
    
    def __pow__(self, exponent):
        if not isinstance(exponent, PariObject):
            exponent = PariObject(exponent)
        return PariObject("v%s ^ v%s"%(self.id, exponent.id))
        



class PariCall:

    def __repr__(self):
        return "PARI"

    def __getattr__(self, cmdname):
        def f(*args):
            cmd = "%s("%cmdname
            for v in args:
                if not isinstance(v, PariObject):
                    x = PariObject(v)
                else:
                    x = v
                cmd += "v%s"%x.id + ","
            cmd = cmd[:-1] + ")"
            return PariObject(cmd)
        self.__dict__[cmdname] = f
        return f

        
pari = PariCall()

#########################################################################
# For doctests

if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
