"""
Big O for various types (power series, p-adics, etc.)
"""

import arith
import laurent_series
import padic_field
import power_series

def O(x):
    if isinstance(x, power_series.PowerSeries):
        return power_series.PowerSeries(x.parent(), x, x.degree())

    elif isinstance(x, laurent_series.LaurentSeries):
        return laurent_series.LaurentSeries(x.parent(), x.unit_part(),
                             x.valuation()).add_bigoh(x.degree())

    elif isinstance(x, (int,long)):  # p-adic number
        if x <= 1:
            raise errors.UsageError("%s must be a prime power >= 2"%x)
        F = arith.factor(x)
        if len(F) != 1:
            raise errors.UsageError("%s must be prime power"%x)
        p, r = F[0]
        return padic_field.Qp(p)(0,r)
    
    raise errors.UsageError("O(%s) not defined"%x)

