"""
Complex numbers
"""

import complex_number
import real_number
import real_field
import ring

_obj_CC = {}
class _uniq_CC(object):
    def __new__(cls):
        if _obj_CC.has_key(0):
            return _obj_CC[0]
        O = object.__new__(cls)
        _obj_CC[0] = O
        return O
    
class ComplexField(_uniq_CC, ring.Ring):
    """
    The field of complex numbers.
        EXAMPLES:
        >>> C = ComplexField(); C
        Complex Field
        >>> C('1/3')
        0.333333333333333333333
        >>> C('1/3', 2)
        0.333333333333333333333 + 2.0*I
        
    Note that the second argument is the number of *bits* of precision,
    not the number of digits of precision:
        >>> C('1/3',2, 100)
        0.3333333333333333333333333333333333333333 + 2.0*I
        
    We can also coerce rational numbers and integers into C, but
    coercing a polynomial in raising an exception.
        >>> Q = RationalField()
        >>> C(Q('1/3'))
        0.333333333333333333333
        >>> C(Q('1/3'), Q('2/3'), 10)
        0.333333333333333333333 + 0.666666666666666666667*I
        >>> S = PolynomialRing(Q)
        >>> C(S.gen())
        Traceback (most recent call last):
        ...
        TypeError: unable to coerce (x,0) to a ComplexNumber
        >>> C.is_field()
        True
        >>> C.characteristic()
        0
        >>> C.name()
        'C'
        >>> C == C
        True
        >>> C == 1.1
        False
    """
    
    def __call__(self, re, im=None, bits=0):
        if isinstance(re, complex_number.ComplexNumber) and (bits == 0 or re.prec() == bits):
            return re
        return complex_number.ComplexNumber(re, im, bits)
    
    def __init__(self):
        ring.Ring.__init__(self)
        
    def __repr__(self):
        return "Complex Field"

    def characteristic(self):
        return 0

    def gen(self, n=0, bits=0):
        if n != 0:
            raise AttributeError, "Generator must be 0"
        return complex_number.ComplexNumber(0,1,bits)
     
    def is_field(self):
        return True
     
    def name(self):
        return "C"
 
    def ngens(self):
        return 1


        
CC = ComplexField()

# doctests
if __name__ ==  '__main__':
    import doctest, sys
    from rational_field import RationalField
    from polynomial_ring import PolynomialRing
    doctest.testmod(sys.modules[__name__])
