"""
Abstract base class for ring elements.
"""

##################################################################
# Generic ring element, so all this functionality must be defined
# by any ring element.  Derived class must call __init__
##################################################################

class Element:
    """
    Generic ring element base class, so all this functionality must
    be defined by any ring element.
    """
    def __init__(self, parent, required_instance=None):
        if required_instance != None:
            assert isinstance(parent, required_instance), \
               "Parent must be of type %s"%required_instance
        self.__parent = parent

    def __repr__(self):
        return "Generic element of a ring"

    def __hash__(self):
        return hash(str(self))

    def order(self):
        raise NotImplementedError

    def is_zero(self):
        return self == self.parent()(0)
    
    def parent(self, x=None):
        if x==None:
            return self.__parent
        else:
            return self.__parent(x)
        
    def __xor__(self, right):
        raise errors.UsageError(\
                "Use ** for exponentiation, not '^', which means xor\n"+\
                "in Python, and has the wrong precedence.")

