"""
Laurent series ring
"""

import weakref

from rational_field import Q
import power_series_ring
import laurent_series

_objsLaurentSeriesRing = {}
class _uniqLaurentSeriesRing(object):
    def __new__(cls, base_ring=Q, variable="q"):
        key = (base_ring, variable)
        if _objsLaurentSeriesRing.has_key(key):
            x = _objsLaurentSeriesRing[key]()
            if x != None: return x
        R = object.__new__(cls)
        _objsLaurentSeriesRing[key] = weakref.ref(R)
        return R

class LaurentSeriesRing(_uniqLaurentSeriesRing, power_series_ring.PowerSeriesRing):
    """
    Univariate Laurent Series Ring

    The Laurent series ring is just pairs a unit polynomial (or 0)
    and an integer (the valuation).  
    """
    def __init__(self, base_ring=Q, variable="q"):
        power_series_ring.PowerSeriesRing.__init__(self, base_ring, variable)
        self._PolynomialRing__generator = laurent_series.LaurentSeries(self, [0,1])
    def __repr__(self):
        return "Laurent series ring in %s over %s"%(self.variable(), self.base_ring())
    def __call__(self, x):
        if isinstance(x, laurent_series.LaurentSeries) and \
               x.parent().base_ring() == self.base_ring():
            return x
        return laurent_series.LaurentSeries(self, x)
    def __cmp__(self, other):
        if not isinstance(other, LaurentSeriesRing):
            return -1
        if self.base_ring() != other.base_ring():
            return -1
        if self.variable() != other.variable():
            return -1
        return 0
