"""
Field Q_p of p-adic numbers.
"""

import weakref

from ring import Ring
from infinity import infinity
import padic

padics = {}
def Qp(p):    
    if padics.has_key(p):
        x = padics[p]()
        if x != None:
            return x
    K = pAdicField(p)
    padics[p] = weakref.ref(K)
    return K

class pAdicField(Ring):
    """
    Field Q_p of p-adic numbers.
    """
    def __init__(self, p):
        Ring.__init__(self)
        self.__p = p
        self.__is_field = True
        self.__prec = 20
        self.__series_print = True
        self.__print_prec = infinity
    def prec(self, n=None):
        if n==None:
            return self.__prec
        self.__prec = n
    def series_print(self, n=None):
        if n==None:
            return self.__series_print
        self.__series_print = n
    def print_prec(self, n=None):
        """If you call print_prec(n), then printing of elements in this
        p-adic field is truncated at O(p^n).   Calling print_prec() with
        no arguments returns n.  This command only affects printing,
        and does not alter the actual values of elements of this field."""
        if n==None:
            return self.__print_prec
        self.__print_prec = n
    def is_field(self):
        """This function always returns true, because Qp is a field.
        >>> K = Qp(7)
        >>> K.is_field()
        True
        """
        return True
    def characteristic(self):
        """The characteristic of the field Qp, which is always 0.
        >>> K = Qp(7)
        >>> K.characteristic()
        0
        """
        return 0
    def prime(self):
        """The prime p such that this is the field Qp.
        >>> K = Qp(7)
        >>> K.prime()
        7
        """
        return self.__p
    def residue_characteristic(self):
        """The characteristic of the residue class field Qp.
        >>> K = Qp(7)
        >>> K.residue_characteristic()
        7
        """
        return self.__p
    def residue_class_field(self):
        """The residue class field of the ring Zp of integers of Qp, i.e., the field Z/pZ.
        >>> K = Qp(3)
        >>> K.residue_class_field()
        Z/3Z
        """
        return IntegerModRing(self.__p)
    def __repr__(self):
        return "Q_%s"%self.__p
    def name(self):
        return "Q_%s"%self.__p
    def __call__(self, x, prec=infinity):
        return padic.pAdic(self, x, prec)
    def __cmp__(self, other):
        if not isinstance(other, pAdicField):
            return -1
        if self.__p < other.__p:
            return -1
        elif self.__p > other.__p:
            return 1
        return 0
    def random(self, prec=10):
        """A random integral element of Q_p to precision O(p**prec)."""
        return pAdic(self, random.randrange(0,self.__p**prec-1), prec)
    
