"""
Ring of pari objects.
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import operator

import sage.pari.py_pari as pari
import sage.rings.ring as ring
import sage.rings.rings 
import sage.rings.element as element
from coerce import bin_op

_obj = {}
class _uniq(object):
    def __new__(cls):
        if _obj.has_key(0):
            return _obj[0]
        O = object.__new__(cls)
        _obj[0] = O
        return O

class PariRing(ring.Ring, _uniq):
    def __contains__(self, x):
        return isinstance(x, Pari)
    
    def __repr__(self):
        return 'Pseudoring of all PARI objects.'

    def __call__(self, x):
        if isinstance(x, Pari):
            return x
        return Pari(x)

    def is_atomic_repr(self):
        return False

    def is_field(self):
        return True

    def name(self):
        return "pari_ring"

    def characteristic(self):
        raise RuntimeError, "Not defined."
        #return 0

    def random(self, bound=0):
        return Pari(0)

    def __cmp__(self, other):
        if isinstance(other, PariRing):
            return 0
        return -1

    def zeta(self):
        return Pari(-1)

_inst = PariRing()

class Pari(element.Element):
    def __init__(self, x):
        self.__x = pari.pari(x)
        
    def parent(self):
        return _inst

    def __repr__(self):
        return str(self.__x)

    def __add__(self, other):
        if not isinstance(other, Pari):
            return bin_op(self, other, operator.add)
        return Pari(self.__x + other.__x)

    def __sub__(self, other):
        if not isinstance(other, Pari):
            return bin_op(self, other, operator.sub)
        return Pari(self.__x - other.__x)

    def __neg__(self):
        return Pari(-self.__x)

    def __mul__(self, other):
        if not isinstance(other, Pari):
            return bin_op(self, other, operator.mul)
        return Pari(self.__x * other.__x)

    def __pow__(self, other):
        if not isinstance(other, Pari):
            return bin_op(self, other, operator.pow)
        return Pari(self.__x ** other.__x)

    def __invert__(self):
        return Pari(~self.__x)

    def __div__(self, other):
        if not isinstance(other, Pari):
            return bin_op(self, other, operator.div)
        return self.__x * (~other.__x)

    def __cmp__(self, other):
        try:
            if not isinstance(other, Pari):
                other = Pari(other)
        except TypeError:
            return -1
        if self.__x < other.__x:
            return -1
        elif self.__x > other.__x:
            return 1
        return 0

    def __int__(self):
        """
        If there is a unique integer in the interval, return it.
        Otherwise raise a ValueError exception.
        """
        return int(self.__x)

            
    
