"""
Univariate Power Series Rings

A power series ring is just a polynomial ring, where we print
out elements backwards and keep track of a precision.
Also, we allow extra operations like exp that wouldn't
otherwise make sense.  Also, the default variable name is q.
"""
from rational_field import Q
import weakref
import polynomial_ring
import power_series

_objsPowerSeriesRing = {}
class _uniqPowerSeriesRing(object):
    def __new__(cls, base_ring=Q, variable="q",):
        key = (base_ring, variable)
        if _objsPowerSeriesRing.has_key(key):
            x = _objsPowerSeriesRing[key]()
            if x != None: return x
        R = object.__new__(cls)
        _objsPowerSeriesRing[key] = weakref.ref(R)
        return R

class PowerSeriesRing(_uniqPowerSeriesRing, polynomial_ring.PolynomialRing):
    def __init__(self, base_ring=Q, variable="q"):
        polynomial_ring.PolynomialRing.__init__(self, base_ring, variable)
        self._PolynomialRing__generator = power_series.PowerSeries(self, [0,1], check=False)

    def __repr__(self):
        return "Power series ring in %s over %s"%(self.variable(), self.base_ring())

    def __call__(self, x):
        if isinstance(x, power_series.PowerSeries) and x.parent() == self:
            return x
        return power_series.PowerSeries(self, x)

    def random(self, degree, bound=10):
        tmp = self([])
        tmp.randomize(degree,bound)
        return tmp

    def __cmp__(self, other):
        if not isinstance(other, PowerSeriesRing):
            return -1
        if self.base_ring() != other.base_ring():
            return -1
        if self.variable() != other.variable():
            return -1
        return 0
