"""
Rings
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

# Abstract base classes
from ring import Ring
from element import Element

# Class Infinity containing the one element infinity
from infinity import Infinity, infinity

# Rational integers.
from integer_ring import IntegerRing
from integer import Integer

# Rational numbers
from rational_field import RationalField
from rational import Rational

# Integers modulo n.
from integer_mod_ring import IntegerModRing, Zmod
from integer_mod import IntegerMod, Mod

# Finite fields
from finite_field import FiniteField, GF
from finite_field_element import FiniteFieldElement

# Number field
from number_field import NumberField, CyclotomicField, QuadraticField
from number_field_element import NumberFieldElement

# p-adic field
from padic_field import pAdicField
from padic import pAdic

# Real numbers
from real_field import RealField_mpf, RealField_decimal
from real_number import RealNumber_mpf, RealNumber_decimal
RealField = RealField_mpf
RealNumber = RealNumber_mpf

# Complex numbers
from complex_field import ComplexField
from complex_number import ComplexNumber

# Univariate Polynomial Rings
from polynomial_ring import PolynomialRing, polygen, polygens
from polynomial import Polynomial
x = PolynomialRing(RationalField()).gen(0)

# Multivariate Polynomial Rings
from multi_polynomial_ring import MPolynomialRing
from multi_polynomial import MPolynomial

# Power series ring in one variable
from power_series_ring import PowerSeriesRing
from power_series import PowerSeries

# Laurent series ring in one variable
from laurent_series_ring import LaurentSeriesRing
from laurent_series import LaurentSeries

# Float interval arithmetic
from interval import IntervalRing, Interval

# Pseudo-ring of PARI objects.
from pari_ring import PariRing, Pari

# Big-oh notation
from big_oh import O

# misc functions
from number_field import is_fundamental_discriminant

##############################################################
# For checking that documentation strings are correct.
##############################################################
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
    



