"""
Base class for objects with generators
"""

#*****************************************************************************
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

class Gens:
    def __getattr__(self, attrname):
        if attrname[0].lower() in ['g', 'x', '_']:
            try:
                if len(attrname)==1:
                    n = 0
                else:
                    n=int(attrname[1:])
            except:
                raise AttributeError, attrname
            if n<0 or n>=self.ngens():
                raise AttributeError, "Generator (=%s) must be between 0 and %s"%\
                      (n,self.ngens()-1)
            return self.gen(n)
        else:
            raise AttributeError, attrname
            
    def __getitem__(self, n):
        return self.list()[n]

    def __getslice__(self, n, m):
        return self.list()[n:m]
    
    def list(self):
        import sage.rings.rings as rings
        try:
            return self.__list
        except AttributeError:
            v = []
            G = self.gens()
            if len(G) == 0:
                self.__list = v
                return
            cnt = [0 for _ in range(len(G))]
            try:
                stop = [g.order()-1 for g in G]
                for x in stop:
                    if x == rings.Infinity():
                        raise ArithmeticError, "%s is not finite."%self
            except AttributeError:
                raise ArithmeticError, "Unable to compute generators of %s"%self
            n = 0
            z = self(1)
            v.append(z)
            while cnt != stop:
                z *= G[0]
                cnt[0] += 1
                i = 0
                while i < len(cnt)-1 and cnt[i]>stop[i]:
                    cnt[i] = 0
                    cnt[i+1] += 1
                    z *= G[i+1]
                    i += 1
                #end
                v.append(z)
            #end
            self.__list = v
        return self.__list
        
    def gens(self):
        return tuple([self.gen(i) for i in range(self.ngens())])

