#!/usr/bin/env python

import os, sys

from distutils.core import setup, Extension

try:
    SAGE_LOCAL = os.environ['SAGE_LOCAL']
except:
    SAGE_LOCAL = "../../../local/"

pyrex_compile_options = ['-Wno-unused-label', '-Wno-unused-function',
                         '-Wno-parentheses',  '-Wno-unused-variable',
                         '-I%s/include'%SAGE_LOCAL]

pyrex_link_options = ['-L%s/lib'%SAGE_LOCAL]

setup(name='sage',
      version='0.3',
      description='SAGE: System for Algebra and Geometry Experimentation',
      author='William Stein',
      author_email='was@math.harvard.edu',
      url='http://modular.fas.harvard.edu/SAGE',
      packages = ['sage',
                  'sage.ext',
                  'sage.algebras',
                  'sage.ellcurve',
                  'sage.functions',
                  'sage.groups',
                  'sage.interface',
                  'sage.linalg',
                  'sage.misc',
                  'sage.modular',
                  'sage.modular.abvar',
                  'sage.modular.modform',
                  'sage.modular.modsym',
                  'sage.modular.ssmod',
                  'sage.pari',
                  'sage.plot',
                  'sage.rings',
                  'sage.tables',
                  'sage.structure'],
      
      ext_modules = [\

         # pari
         Extension('sage.ext._py_pari',
                   ['sage/ext/_py_pari.c'],
                   libraries=['pari'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         # rings
         Extension('sage.ext._arith',
                   ['sage/ext/_arith.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         Extension('sage.ext._arith_gmp',
                   ['sage/ext/_arith_gmp.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.ext._element',
                   ['sage/ext/_element.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.ext._integer',
                   ['sage/ext/_arith.c',
                    'sage/ext/_integer.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.ext._intmod_pyx',
                   ['sage/ext/_intmod_pyx.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.ext._polynomial_pyx',
                   ['sage/ext/_polynomial_pyx.c',
                    'sage/ext/_arith_gmp.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         Extension('sage.ext._rational',
                   ['sage/ext/_rational.c',
                    'sage/ext/_arith.c', \
                    'sage/ext/_integer.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.ext._sparse_poly',
                   ['sage/ext/_sparse_poly.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         # linalg
         Extension('sage.ext._sparse_matrix_pyx',
                   ['sage/ext/_sparse_matrix_pyx.c',
                    'sage/ext/_integer.c',
                    'sage/ext/_rational.c',
                    'sage/ext/_arith.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \
         
         Extension('sage.ext._dense_matrix_pyx',
                   ['sage/ext/_dense_matrix_pyx.c',
                    'sage/ext/_integer.c',
                    'sage/ext/_rational.c',
                    'sage/ext/_arith.c'],
                   libraries=['gmp'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         # Search
         Extension('sage.ext._search',
                   ['sage/ext/_search.c'],
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         # modular forms
         Extension('sage.ext._heilbronn',
                   ['sage/ext/_heilbronn.c',
                    'sage/ext/_p1list.c',
                    'sage/ext/_arith.c'],
                   libraries = ['gmp', 'm'], 
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options), \

         Extension('sage.ext._p1list',
                   ['sage/ext/_p1list.c',
                    'sage/ext/_arith.c'],
                   libraries = ['gmp'], 
                   extra_compile_args = pyrex_compile_options,
                   extra_link_args = pyrex_link_options)
         ]
     )
 
 
